package com.example.mchart;

import android.app.Activity;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.graphics.drawable.BitmapDrawable;
import android.os.Handler;
import android.os.Bundle;
import android.util.Log;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.supermap.data.CursorType;
import com.supermap.data.DatasetVector;
import com.supermap.data.Environment;
import com.supermap.data.Point2D;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.Layer;
import com.supermap.mapping.Map;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.MapView;
import com.supermap.mapping.dyn.DynamicElement;
import com.supermap.mapping.dyn.DynamicPoint;
import com.supermap.mapping.dyn.DynamicStyle;
import com.supermap.mapping.dyn.DynamicView;
import com.supermap.mapping.dyn.TranslateAnimator;
import com.supermap.mapping.imChart.ChartPoint;
import com.supermap.mapping.imChart.ChartView;
import com.supermap.mapping.imChart.GridHotChart;
import com.supermap.mapping.imChart.*;
import com.supermap.mapping.imChart.PointDensityChart;
import com.supermap.mapping.imChart.PolymerChart;
import com.supermap.mapping.imChart.RelationPointChart;
import com.supermap.mapping.imChart.RelationalChartPoint;
import com.supermap.mapping.imChart.TimeLine;
import com.supermap.mapping.view.LayerListView;

import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


public class MainActivity extends Activity {

    private Workspace workspace;
    private MapView mMapView;
    private HeatMap mHotChart;
    private PolymerChart mPolymerChart;
    private GridHotChart mGridHotChart;
    private PointDensityChart mDensithart;
    private RelationPointChart mRelationChart;
    Map map;
    private Button mBtnRealTime,mTimeData;
    private boolean bDataInit = false;
    private Handler mHandle = new Handler()
    {
        public void handleMessage(android.os.Message msg) {
            if(msg.what == 1000001) {
                LinearLayout layout = (LinearLayout) findViewById(R.id.layout_left);
                layout.setVisibility(View.VISIBLE);
                Button btn = (Button) findViewById(R.id.btn_hot);
                btn.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        clear();
                        initHotMap();
                    }
                });

                btn = (Button) findViewById(R.id.btn_polymer);
                btn.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        clear();
                        initPolymeMap();
                    }
                });
                btn = (Button) findViewById(R.id.btn_density);
                btn.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        clear();
                        initDensityPoint();
                    }
                });
                btn = (Button) findViewById(R.id.btn_relative);
                btn.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        clear();
                        initRelationMap();
                    }
                });

                mBtnRealTime = (Button) findViewById(R.id.btn_real_time);
                mBtnRealTime.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {

                        boolean bSelected = false;
                        if(!mBtnRealTime.isSelected()){
                            bSelected = true;
                        }
                        mBtnRealTime.setSelected(bSelected);
                       realTimeData(bSelected);
                    }
                });

                mTimeData = (Button) findViewById(R.id.btn_time_data);
                mTimeData.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                       // clear();
                        //// TODO: 17/5/11 timedata
                        timeData();
                    }
                });

                btn = (Button) findViewById(R.id.btn_grid);
                btn.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        clear();
                        initGridHotMap();
                    }
                });

                Toast toast = Toast.makeText(getApplicationContext(), "The data is loaded...", Toast.LENGTH_SHORT);
                toast.setGravity(Gravity.CENTER,0,0);
                toast.show();

                ProgressBar progressBar = (ProgressBar) findViewById(R.id.progressBar);
                progressBar.setVisibility(View.GONE);
            }
        };
    };
    private void openMap(){
        String rootPath =android.os.Environment.getExternalStorageDirectory().getAbsolutePath();
        Environment.initialization(this);
        Environment.setOpenGLMode(true);
        Environment.setLicensePath(rootPath + "/SuperMap/license");
        String jStirng = Environment.getLicensePath();


        setContentView(R.layout.activity_main_chart);
        RelativeLayout view  = (RelativeLayout) findViewById(R.id.activity_main);

        mMapView = new MapView(this);
        //mMapView.setLayoutParams(new ViewGroup.LayoutParams(1000,1000));

        map = mMapView.getMapControl().getMap();
        workspace = map.getWorkspace();
        
        String path =  rootPath + "/SampleData/VisualizationData/hotMap.smwu";
        WorkspaceConnectionInfo info = new WorkspaceConnectionInfo();
        info.setServer(path);
        info.setType(WorkspaceType.SMWU);
        boolean isOpen = workspace.open(info);
        if(isOpen){
            map.open(workspace.getMaps().get(0));
         //   map.viewEntire();
            map.refresh();
        }else {
            Toast.makeText(this, "Open workspace failed!", Toast.LENGTH_LONG).show();
        }
        info.dispose();

        map.refresh();
        view.addView(mMapView,0);
        
    }
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if(!bDataInit) {
            openMap();
            ExecutorService mFixedThreadPool = Executors.newFixedThreadPool(1);
            mFixedThreadPool.execute(new Runnable() {
                @Override
                public void run() {
                    prepateData();
                    mHandle.obtainMessage(1000001).sendToTarget();
                }
            });
            Toast toast = Toast.makeText(getApplicationContext(), "Loading...", Toast.LENGTH_LONG);
            toast.setGravity(Gravity.BOTTOM,0,0);
            toast.show();
            bDataInit = true;
        }
        mMapView.setBackgroundColor(0x0);

        FrameLayout frameLayoutLayout = (FrameLayout) findViewById(R.id.layout_timeline);
        frameLayoutLayout.setVisibility(View.INVISIBLE);
    }

    private ArrayList<ChartPoint> mHotDatas = new ArrayList<ChartPoint>();
    private ArrayList<RelationalChartPoint> mRelativeDatas = new ArrayList<RelationalChartPoint>();
    private ArrayList<ChartPoint> m_30WData = new ArrayList<ChartPoint>();
    private ArrayList<ChartPoint> mPolymerDatas = new ArrayList<ChartPoint>();
    private void prepateData(){
        Recordset rd = ((DatasetVector)workspace.getDatasources().get(0).getDatasets().get(3)).getRecordset(false, CursorType.DYNAMIC);

        Random rand = new Random();
        int   i = 0;
        //hot
        int n = rd.getRecordCount();
        for(int k=0;k<5000;k++){
           int index = rand.nextInt(n);
            rd.moveTo(index);
            Point2D point = rd.getGeometry().getInnerPoint();
            float dw = rand.nextInt(100)+1;
            mHotDatas.add(new ChartPoint(point,dw));
        }
//        while (!rd.isEOF()) {
//
//            Point2D point = rd.getGeometry().getInnerPoint();
//            float dw = rand.nextInt(50)+1;
//            mHotDatas.add(new ChartPoint(point,dw));
//            rd.moveNext();
//            if(++i>5000)
//                break;
//        }
        rd.dispose();


        //polymer
        rd = ((DatasetVector)workspace.getDatasources().get(0).getDatasets().get(6)).getRecordset(false, CursorType.DYNAMIC);

        while (!rd.isEOF()) {

            Point2D point = rd.getGeometry().getInnerPoint();
            mPolymerDatas.add(new ChartPoint(point,0));
            rd.moveNext();
        }
        rd.dispose();

        //relative
        rd = ((DatasetVector)workspace.getDatasources().get(0).getDatasets().get(0)).getRecordset(false, CursorType.DYNAMIC);

        while (!rd.isEOF()) {

            Point2D point = rd.getGeometry().getInnerPoint();
            RelationalChartPoint relPoint = new RelationalChartPoint(point,0);
            relPoint.setRelationName((String)rd.getFieldValue("name"));

            mRelativeDatas.add(relPoint);
            rd.moveNext();
        }
        rd.dispose();


        //grid,density
        rd = ((DatasetVector)workspace.getDatasources().get(0).getDatasets().get(5)).getRecordset(false, CursorType.DYNAMIC);
        i = 0;
        while (!rd.isEOF()) {

            Point2D point = rd.getGeometry().getInnerPoint();
            m_30WData.add(new ChartPoint(point,0));
            rd.moveNext();
            if(++i>15000)
                break;
        }
        rd.dispose();
    }
    private Timer realTimer = null;
    private void realTimeData(boolean bSelected){

        if(timeLine != null){
            timeLine.dispose();
            timeLine = null;
            mHotChart.removeAllData();
        }
        Log.v("xzy","~~~~~~~stop");
        if(bSelected) {
            if (realTimer == null) {
                realTimer = new Timer();
            } else
                realTimer.cancel();
        }else {
            if (realTimer != null) {
                realTimer.cancel();
                realTimer = null;
                mHotChart.setUpdataInterval(0);
                return;
            }
        }
        realTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                Random rand = new Random();
                int nCount = 5000 + rand.nextInt(mHotDatas.size()+5000);
                ArrayList<ChartPoint> arrTmp = new ArrayList<ChartPoint>();
                for(int i=0;i<nCount;i++){

                    int index = rand.nextInt(mHotDatas.size());

                    {
                        ChartPoint weightPoint = mHotDatas.get(index);
                        float dWeight = rand.nextInt(50)+1;
                        weightPoint.setWeighted(dWeight);

                        arrTmp.add(weightPoint);

                    }
                }
                mHotChart.addChartDatas(arrTmp);
            }
        },50,2000);
        mHotChart.setUpdataInterval(2.0f);
    }

    private  TimeLine timeLine;
    private void timeData(){
        mBtnRealTime.setSelected(false);
        mHotChart.removeAllData();

        mHotChart.setUpdataInterval(0);
        if(realTimer != null){
            realTimer.cancel();
            realTimer = null;
        }

        Random rand = new Random();

        ArrayList<Point2D> targetCitys = new ArrayList<Point2D>();
        int[] speeds = new int[mHotDatas.size()+1];
        int nSteps = 16;
        for(int i=0;i<mHotDatas.size();i++){

            int speed = rand.nextInt(nSteps)+1;
            int city = rand.nextInt(10);
            if(city<4){//bj
                targetCitys.add(new Point2D(116.26,39.56));
            }else if(city>=4 && city<7){//sh
                targetCitys.add(new Point2D(121.27,31.13));
            }else if(city>=7 && city<9){//gz
                targetCitys.add(new Point2D(114.6,22.59));
            }else{//cd
                targetCitys.add(new Point2D(103.59,30.45));
            }
            speeds[i] = speed;
        }

        for(int j=0;j<nSteps;j++){
            ArrayList<ChartPoint>  arrTmp = new ArrayList<ChartPoint>();
            for(int i=0;i<mHotDatas.size();i++){

                ChartPoint weightPoint = mHotDatas.get(i);
                int speed = speeds[i];
                Point2D targetCity = targetCitys.get(i);

                float dWeight = 25 ;//+ rand.nextInt(20);

                double offsetX = (targetCity.getX()-weightPoint.getPoint().getX());
                double offsetY = (targetCity.getY()-weightPoint.getPoint().getY());

                float n = j;
                if(j > speed){
                    n = speed;
                }
                if(j==0)
                    n = 0.10f;
                double dX = weightPoint.getPoint().getX()+(offsetX/speed)*n;
                double dY = weightPoint.getPoint().getY()+(offsetY/speed)*n;
                arrTmp.add(new ChartPoint(new Point2D(dX,dY),dWeight));
            }
            mHotChart.insertChartDataset(arrTmp,String.valueOf(nSteps-j)+"h",0);
        }

        mHotChart.setPlayInterval(1.0f);
        mHotChart.setIsLoopPlay(true);


        FrameLayout frameLayoutLayout = (FrameLayout) findViewById(R.id.layout_timeline);
        frameLayoutLayout.setVisibility(View.VISIBLE);
        timeLine = new TimeLine(frameLayoutLayout,this);
        timeLine.addChart(mHotChart);
        timeLine.load();

        mHotChart.startPlay();
    }
    private void initRelationMap()
    {

        for(int i=0;i<map.getLayers().getCount();i++){
            Layer layer = map.getLayers().get(i);
            layer.setVisible(false);
        }
        Layer layer1 = map.getLayers().get("China_line_vac@hotMap");
        layer1.setVisible(true);
        Layer layer = map.getLayers().get("Provinces_R@hotMap#2");
        layer.setVisible(true);

        map.setScale(0.000000081887287763467158);
        map.setCenter(new Point2D(109.38087581861375,30.922633709727265));
        map.refresh();

        ArrayList<RelationalChartPoint> datas = new ArrayList<RelationalChartPoint>();
        ArrayList<RelationalChartPoint> targetArr = new ArrayList<RelationalChartPoint>();

        //  Recordset rd = ((DatasetVector)workspace.getDatasources().get(0).getDatasets().get(0)).getRecordset(false, CursorType.DYNAMIC);

        for(int i = 0;i<mRelativeDatas.size();i++)
        {
            RelationalChartPoint relPoint = mRelativeDatas.get(i);
            relPoint.getRelationalPoints().clear();
            String name = relPoint.getRelationName();
            if(
                    name.equalsIgnoreCase("成都市") ||
                            name.equalsIgnoreCase("北京市") ||
                            name.equalsIgnoreCase("上海市") ||
                            name.equalsIgnoreCase("广州市") ||
                            name.equalsIgnoreCase("乌鲁木齐市") ||
                            name.equalsIgnoreCase("长春市") ||
                            name.equalsIgnoreCase("长沙市") ||
                            name.equalsIgnoreCase("呼和浩特市") ||
                            name.equalsIgnoreCase("大理市")

                    ){
                targetArr.add(relPoint);
            }
            datas.add(relPoint);
        }

        int n = datas.size();
        for(int j=0;j<targetArr.size();j++){
            RelationalChartPoint relPoint = targetArr.get(j);
            for(int k=0;k<targetArr.size();k++)
            {
                RelationalChartPoint relPoint1 = targetArr.get(k);
                if(relPoint1.getRelationName().equalsIgnoreCase("北京市")){
                    relPoint1.setWeighted(50);
                }else if(relPoint1.getRelationName().equalsIgnoreCase("上海市")){
                    relPoint1.setWeighted(45);
                }else if(relPoint1.getRelationName().equalsIgnoreCase("广州市")){
                    relPoint1.setWeighted(36);
                }else if(relPoint1.getRelationName().equalsIgnoreCase("成都市")){
                    relPoint1.setWeighted(40);
                }else if(relPoint1.getRelationName().equalsIgnoreCase("长春市")){
                    relPoint1.setWeighted(25);
                }else if(relPoint1.getRelationName().equalsIgnoreCase("长沙市")){
                    relPoint1.setWeighted(50);
                }

                if(relPoint1.getRelationName().equals(relPoint.getRelationName())==false){
                    relPoint.getRelationalPoints().add(relPoint1);
                }
            }
        }

        Resources res = MainActivity.this.getResources();
        Bitmap bmp= BitmapFactory.decodeResource(res, R.drawable.icon_plan);

        mRelationChart = new RelationPointChart(this,mMapView);
        mRelationChart.setIsAnimation(true);
        mRelationChart.setLineWidth(0.75f);
        mRelationChart.setMaxRadious(30);
        mRelationChart.setAnimationImage(bmp);
        mRelationChart.addChartRelationDatas(targetArr);
        mRelationChart.setTitle("Relationship Map");

    }
    private void initPolymeMap(){
        for(int i=0;i<map.getLayers().getCount();i++){
            Layer layer = map.getLayers().get(i);
            layer.setVisible(false);
        }
        Layer layer1 = map.getLayers().get("China_line_vac@hotMap");
        layer1.setVisible(true);
        Layer layer = map.getLayers().get("Provinces_R@hotMap#2");
        layer.setVisible(true);

        map.setScale(0.00000004710369833422136);
        map.setCenter(new Point2D(104.45644039125189,34.960112480819163));
        map.refresh();

        mPolymerChart = new PolymerChart(this,mMapView);
        mPolymerChart.setMaxRadious(45);
        mPolymerChart.addChartDatas(mPolymerDatas);
        mPolymerChart.setTitle("Polymer Map");
    }
    private void initGridHotMap(){
        for(int i=0;i<map.getLayers().getCount();i++){
            Layer layer = map.getLayers().get(i);
            layer.setVisible(false);
        }
        Layer layer1 = map.getLayers().get("China_line_vac@hotMap");
        layer1.setVisible(true);
        Layer layer = map.getLayers().get("Provinces_R@hotMap#2");
        layer.setVisible(true);

        map.setScale(3.4070912983014954E-8);
        map.setCenter(new Point2D(112.00259362778627,36.26642049416556));
        map.refresh();

        mGridHotChart = new GridHotChart(this,mMapView);
        mGridHotChart.addChartDatas(m_30WData);
        mGridHotChart.setTitle("Grid Heat Map");
    }
    private void initDensityPoint(){

        for(int i=0;i<map.getLayers().getCount();i++){
            Layer layer = map.getLayers().get(i);
            layer.setVisible(false);
        }
        Layer layer1 = map.getLayers().get("China_line_vac@hotMap");
        layer1.setVisible(true);
        Layer layer = map.getLayers().get("Provinces_R@hotMap");
        layer.setVisible(true);

        map.setScale(0.000000065099303341972671);
        map.setCenter(new Point2D(108.79366417027659,32.180036928883418));
        map.refresh();

        mDensithart = new PointDensityChart(this,mMapView);
        mDensithart.setRadious(3);
        mDensithart.addChartDatas(m_30WData);
        mDensithart.setTitle("Density Map");
    }
    private void initHotMap(){

        LinearLayout layout = (LinearLayout) findViewById(R.id.layout_right);
        layout.setVisibility(View.VISIBLE);

        for(int i=0;i<map.getLayers().getCount();i++){
            Layer layer = map.getLayers().get(i);
            layer.setVisible(false);
        }
        Layer layer1 = map.getLayers().get("China_line_vac@hotMap");
        layer1.setVisible(true);
        Layer layer = map.getLayers().get("Provinces_R@hotMap");
        layer.setVisible(true);

        map.setScale(0.000000065099303341972671);
        map.setCenter(new Point2D(108.79366417027659,32.180036928883418));
        map.refresh();

        mHotChart = new HeatMap(this,mMapView);
        mHotChart.setRadious(13);
        mHotChart.setSmoothTransColor(true);
        mHotChart.addChartDatas(mHotDatas);
        mHotChart.setTitle("Heat Map");
    }

    private void clear(){
        if(mPolymerChart != null)
            mPolymerChart.dispose();
        if(mHotChart != null)
            mHotChart.dispose();
        if(mDensithart != null)
            mDensithart.dispose();
        if(mRelationChart != null)
            mRelationChart.dispose();
        if(mGridHotChart != null)
            mGridHotChart.dispose();

        mPolymerChart = null;
        mHotChart =null;
        mDensithart = null;
        mRelationChart = null;
        mGridHotChart = null;

        if(realTimer != null){
            realTimer.cancel();
            realTimer = null;
        }
        if(timeLine != null){
            timeLine.dispose();
            timeLine = null;
        }
        FrameLayout frameLayoutLayout = (FrameLayout) findViewById(R.id.layout_timeline);
        frameLayoutLayout.setVisibility(View.INVISIBLE);

        mBtnRealTime.setSelected(false);
        LinearLayout layout = (LinearLayout) findViewById(R.id.layout_right);
        layout.setVisibility(View.INVISIBLE);
    }
}
